VM_SECURITY_CHECKS = "vm.security_checks"
VM_SYNC_SCREENSHOT = "vm.sync_screenshot"
VM_STATE_CHANGED = "vm.state_changed"
VM_RUN_UPDATE = "vm.run_update"
VM_HEALTH_CHECK = "vm.health_check"
VM_REBOOT = "vm.reboot"
VM_DISABLE_HEALTH_CHECK_TEST = "vm.disable_health_check_test"
VM_SEND_REPORT = "vm.send_report"
VM_STATE_STARTED = "vm.state.started"
VM_STATE_STOPPED = "vm.state.stopped"
VM_STATE_SUSPENDED = "vm.state.suspended"
VM_STATE_RESUMED = "vm.state.resumed"
VM_STATE_PAUSED = "vm.state.paused"
VM_STATE_CHANGED = "vm.state.changed"
PACKAGE_UPDATE = "package.update"
SYSTEM_UPDATE = "system.update"
SYSTEM_ERROR = "system.error"
SYSTEM_WARNING = "system.warning"
SYSTEM_INFO = "system.info"
SYSTEM_DEBUG = "system.debug"
SYSTEM_TRACE = "system.trace"
SYSTEM_CRITICAL = "system.critical"
SYSTEM_RESTART_BACKEND = "system.restart_backend"
CHAT_CLEAR_HISTORY = "chat.clear_history"
CHAT_CLEAR_CONTEXT = "chat.clear_context"
CHAT_CLEAR_SESSION = "chat.clear_session"
CHAT_CLEAR_ALL = "chat.clear_all"
CHAT_CLEAR_ALL_MESSAGES = "chat.clear_all_messages"
CHAT_CLEAR_ALL_MESSAGES = "chat.clear_all_messages"
CONFIG_SETTING_CHANGED = "config.setting_changed"
